<?php

namespace PeytzMail\Tests;

class TestCase extends \PHPUnit_Framework_TestCase
{
    /**
     * @return Client
     */
    protected function getStubClient()
    {
        return $this->getMock('\\PeytzMail\\Client');
    }

    /**
     * @param  string $returnValue
     * @return Client
     */
    protected function getMockClientReturns($returnValue)
    {
        $mock = $this->getMock('\\PeytzMail\\Client');
        $mock
            ->expects($this->once())
            ->method('getContent')
            ->will($this->returnValue($returnValue));

        return $mock;
    }
}
