<?php

namespace PeytzMail\Tests;

use PeytzMail\Config;

class ConfigTest extends TestCase
{
    protected $config;

    protected function setUp()
    {
        $this->config = new Config('foo', 'bar');
    }

    public function testEndPoint()
    {
        $this->assertSame('https://foo.peytzmail.com', $this->config->getEndPoint());
    }

    public function testGetApiKey()
    {
        $this->assertSame('bar', $this->config->getApiKey());
    }
}
