<?php

namespace PeytzMail;

/**
 * @package PeytzMail
 */
class Config
{
    const ENDPOINT = 'https://%s.peytzmail.com';

    protected $endPoint;
    protected $apiKey;

    /**
     * @param string $customerName
     * @param string $apiKey
     */
    public function __construct($customerName, $apiKey)
    {
        $this->endPoint = sprintf(self::ENDPOINT, $customerName);
        $this->apiKey = $apiKey;
    }

    /**
     * @return string
     */
    public function getEndPoint()
    {
        return $this->endPoint;
    }

    /**
     * @return string
     */
    public function getApiKey()
    {
        return $this->apiKey;
    }
}
